@@ -24,13 +24,7 @@ use Scalar::Util qw/weaken/;
 
 =head1 NAME
 
-Games::Construder::Server::Objects - desc
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-=head1 METHODS
+Games::Construder::Server::Objects - Implementation of Object Type specific behaviour
 
 =over 4
 
@@ -358,7 +352,9 @@ sub drone_check_player_hit {
 
    if (vlength (vsub ($pl->{data}->{pos}, $pos)) <= 1.1) {
       my $dist = $entity->{teleport_dist} * 60;
-      my $new_pl_pos = vsmul (vnorm (vrand ()), $dist);
+      my ($new_pl_pos, $dist, $secdist) =
+         world_find_random_teleport_destination_at_dist ($pl->{data}->{pos}, $dist);
+      $dist = int $dist;
       $pl->teleport ($new_pl_pos);
       $pl->push_tick_change (happyness => -100);
       $pl->msg (1, "A Drone displaced you by $dist.");
@@ -717,8 +713,6 @@ sub tmr_auto {
 
 Robin Redeker, C<< <elmex@ta-sa.org> >>
 
-=head1 SEE ALSO
-
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2011 Robin Redeker, all rights reserved.
