@@ -22,6 +22,7 @@ use Games::Construder::Server::World;
 use Games::Construder::Server::UI;
 use Games::Construder::Server::Objects;
 use Games::Construder::Server::PatStorHandle;
+use Games::Construder::Logging;
 use Games::Construder::Vector;
 use Time::HiRes qw/time/;
 use base qw/Object::Event/;
@@ -30,22 +31,15 @@ use Compress::LZF;
 
 =head1 NAME
 
-Games::Construder::Server::Player - desc
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-=head1 METHODS
+Games::Construder::Server::Player - Implementation of Player central Game Mechanics (the heart of the game)
 
 =over 4
 
-=item my $obj = Games::Construder::Server::Player->new (%args)
-
 =cut
 
 my $PL_VIS_RAD = 3;
 my $PL_MAX_INV = 24;
+my $PL_MAX_QUEUE_SIZE = 100; # max 100 chunks
 
 sub new {
    my $this  = shift;
@@ -69,14 +63,14 @@ sub _check_file {
       my $cont = do { local $/; <$plf> };
       my $data = eval { JSON->new->relaxed->utf8->decode ($cont) };
       if ($@) {
-         warn "Couldn't parse player data from file '$file': $!\n";
+         ctr_log (error => "Couldn't parse player data from file '%s': %s", $file, $!);
          return;
       }
 
       return $data
 
    } else {
-      warn "Couldn't open player file $file: $!\n";
+      ctr_log (error => "Couldn't open player file '%s': %s", $file, $!);
       return;
    }
 }
@@ -89,7 +83,10 @@ sub _initialize_player {
       happyness => 100,
       bio       => 100,
       score     => 0,
-      pos       => [0, 0, 0],
+ #     pos       => [1.1 * 60, -29 * 60, 1.1 * 60],
+      pos       => [
+         map { 60 * $_ } $Games::Construder::Server::RES->get_initial_position
+      ],
       time      => 0,
       inv       => $inv,
       next_encounter => 15 * 60, # 15 minutes newbie safety
@@ -125,19 +122,19 @@ sub save {
       close $plf;
 
       if (-s "$file~" != length ($cont)) {
-         warn "Couldn't write out player file completely to '$file~': $!\n";
+         ctr_log (error => "Couldn't write out player file completely to '%s': %s", $file, $!);
          return;
       }
 
       unless (rename "$file~", "$file") {
-         warn "Couldn't rename $file~ to $file: $!\n";
+         ctr_log (error => "Couldn't rename $file~ to $file: $!");;
          return;
       }
 
-      warn "saved player $self->{name} to $file.\n";
+      ctr_log (info => "Saved player %s to %s", $self->{name}, $file);
 
    } else {
-      warn "Couldn't open player file $file~ for writing: $!\n";
+      ctr_log (error => "Couldn't open player file $file~ for writing: $!");
       return;
    }
 }
@@ -225,7 +222,7 @@ sub init {
       if ($wself->{uis}->{inventory}->{shown}) {
          $wself->{uis}->{inventory}->show;
       }
-      warn "INVENTORY CHANGED!\n";
+
       $wself->{uis}->{slots}->show;
    });
 
@@ -351,6 +348,7 @@ sub try_eat_something {
       if ($item->[1] <= $amount) {
          my ($ov) = $self->{inv}->remove ($item->[0]);
          if ($ov) {
+            $self->{uis}->{status}->show ([@$item]);
             $self->refill_bio ($item->[1]);
             return 1;
          }
@@ -359,6 +357,7 @@ sub try_eat_something {
    } else {
       my ($ov) = $self->{inv}->remove ($item->[0]);
       if ($ov) {
+         $self->{uis}->{status}->show ([@$item]);
          $self->refill_bio ($item->[1]);
          return 1;
       }
@@ -381,9 +380,10 @@ sub refill_bio {
 
 sub kill_player {
    my ($self, $reason) = @_;
-   my $new_pl_pos = vsmul (vnorm (vrand ()), 780);
-   $self->teleport ($new_pl_pos);
-   $self->msg (1, "You died of $reason, your stats and inventory were reset and you have been teleported 13 sectors away!");
+   my $pp = $self->{data}->{pos};
+   my ($new_pos, $dist, $secdist) = world_find_random_teleport_destination_at_dist ($pp, 780);
+   $self->teleport ($new_pos);
+   $self->msg (1, "You died of $reason, your stats and inventory were reset and you have been teleported $secdist sectors away!");
    $self->{inv}->remove ('all');
    my $inv = $Games::Construder::Server::RES->get_initial_inventory;
    $self->{data}->{inv}->{$_} = $inv->{$_} for keys %$inv;
@@ -400,7 +400,8 @@ sub logout {
    delete $self->{upd_score_hl_tmout};
    delete $self->{death_timer};
    delete $self->{tick_timer};
-   warn "player $self->{name} logged out\n";
+
+   ctr_log (info => "Player %s logged out", $self->{name});
    #d# print Devel::FindRef::track $self;
 }
 
@@ -410,7 +411,7 @@ sub update_pos {
    my ($self, $pos, $lv) = @_;
 
    if ($self->{freeze_update_pos} ne '') {
-      warn "update_pos thrown away, awaiting teleport confirmation!\n";
+      ctr_log (debug => "update_pos thrown away, awaiting teleport confirmation!");
       return;
    }
 
@@ -536,8 +537,11 @@ sub push_chunk_to_network {
       vlength (vsub ($plchnk, $b))
    } @upds;
 
+   splice @upds, $PL_MAX_QUEUE_SIZE;
+
    my $cnt = scalar @upds;
-   print "$cnt chunk upodates in queue!\n";
+   ctr_log (debug => "player %s: %d chunk updates in queue", $self->{name}, $cnt)
+      if $cnt;
    for (my $i = 0; $i < 5; $i++) {
       my $q = shift @upds
          or return;
@@ -592,7 +596,6 @@ sub interact {
 
    world_at ($pos, sub {
       my ($pos, $cell) = @_;
-      print "interact position [@$pos]: @$cell\n";
       Games::Construder::Server::Objects::interact ($self, $pos, $cell->[0], $cell->[5]);
    });
 }
@@ -625,7 +628,7 @@ sub debug_at {
    });
    world_mutate_at ($pos, sub {
       my ($data) = @_;
-      print "position [@$pos]: @$data\n";
+      ctr_log (debug => "player %s: debug position @$pos: @$data", $self->{name});
       if ($data->[0] == 1) {
          $data->[0] = 0;
          return 1;
@@ -906,7 +909,7 @@ sub take_assignment {
    my $vec  = vsmul (vnorm (vrand ()), $offer->{distance});
    my $wpos = vfloor (vadd ($vec, $self->get_pos_normalized));
 
-   warn "assignment at @$vec => @$wpos\n";
+   ctr_log (debug => "took assignment at @$vec => @$wpos");
 
    my $size = $offer->{size};
    Games::Construder::VolDraw::alloc ($size);
@@ -944,7 +947,6 @@ sub take_assignment {
    $cal->{pos}        = $wpos;
    $cal->{materials}  = [sort keys %$materials];
    $cal->{sel_mat}    = $cal->{materials}->[0];
-   print "ASSIGNMENT : " . JSON->new->pretty->encode ($cal) . "\n";
    $cal->{pos_types}  = $positions;
    $cal->{mat_models} = $materials;
 
@@ -1107,7 +1109,7 @@ sub create_encounter {
    my ($teledist, $nxttime, $lifetime) =
       $Games::Construder::Server::RES->encounter_values ();
 
-   warn "NEXT ENC $nxttime ($teledist, $lifetime)\n";
+   ctr_log (debug => "next encounter for player %s is %d (%d, %d)", $self->{name}, $nxttime, $teledist, $lifetime);
    $self->{data}->{next_encounter} = $nxttime;
 
    world_mutate_at ($new_pos, sub {
@@ -1159,17 +1161,15 @@ sub teleport {
    my ($self, $pos) = @_;
 
    $pos ||= $self->{data}->{pos};
-   warn "START TELEPORT @$pos\n";
    $self->msg (0, "Teleport in progress, please wait...");
    world_load_around_at ($pos, sub {
-      warn "TELEPORT @$pos\n";
       my $new_pos = world_find_free_spot ($pos, 1);
-      unless ($new_pos) {
+      unless (@$new_pos) {
          $new_pos = world_find_free_spot ($pos, 0); # without floor on second try
       }
 
       unless (@$new_pos) {
-         warn "new position for player at @$pos had no free spot! moving him up!\n";
+         ctr_log (error => "new position for player at @$pos had no free spot! moving him up!");
          viaddd ($pos, 0, 10, 0);
          my $t; $t = AE::timer 0, 0, sub {
             $self->teleport ($pos);
@@ -1177,16 +1177,13 @@ sub teleport {
          };
       }
 
-      warn "FREESPOT @$new_pos\n";
       $new_pos = vaddd ($new_pos, 0.5, 0.5, 0.5);
-      warn "FREESPOT AT EXACTLY @$new_pos\n";
       my $fid = "$new_pos";
       $self->{data}->{pos} = $new_pos;
       $self->{freeze_update_pos} = $fid;
       $self->upd_visible_chunks;
       $self->send_client ({ cmd => "place_player", pos => $new_pos, id => $fid });
    });
-   warn "END TELEPORT @$pos\n";
 }
 
 sub unfreeze_update_pos {
@@ -1224,7 +1221,8 @@ sub display_ui {
 
 sub ui_res : event_cb {
    my ($self, $ui, $cmd, $arg, $pos) = @_;
-   warn "ui response $ui: $cmd ($arg) (@$pos)\n";
+
+   ctr_log (debug => "player %s: ui response %s: %s (%s) (@$pos)", $ui, $cmd, $arg);
 
    if (my $o = $self->{uis}->{$ui}) {
       $o->react ($cmd, $arg, $pos);
@@ -1236,7 +1234,7 @@ sub ui_res : event_cb {
 
 sub DESTROY {
    my ($self) = @_;
-   warn "player $self->{name} [$self] destroyed!\n";
+   ctr_log (debug => "player %s [%s] destroyed", $self->{name}, $self);
 }
 
 =back
@@ -1245,8 +1243,6 @@ sub DESTROY {
 
 Robin Redeker, C<< <elmex@ta-sa.org> >>
 
-=head1 SEE ALSO
-
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2011 Robin Redeker, all rights reserved.
